//+------------------------------------------------------------------+
//|                                              NewsTrackerV1_1.mqh |
//|                                               Paul Hampton-Smith |
//|                                               paul1000@pobox.com |
//+------------------------------------------------------------------+
/*


Copy this include file to the experts\include folder. No need to compile

This include file is intended for the following two uses:

EITHER
~~~~~~
to be used with systems that work on the shorter timeframes (1M, 5M and 15M) to 
give them the opportunity to be out of the market during news events.

example:
   #include <NewsTrackerV1_1.mqh>

   string strNews; 
   datetime NewsTime;

   // avoid buying if within 5 minutes of news event
   if (MyBuySignal && !NewsEvent(5,strNews,NewsTime)) 
      DoMyBuying();

   // close all positions if within 5 minutes of news event
   if (OpenOrders() && NewsEvent(5,strNews,NewsTime)) 
      CloseOrDeleteAllOrders();

 OR
~~~~
to be used with systems that work on the news event itself
   
   #include <NewsTrackerV1_1.mqh>

   string strNews; 
   datetime NewsTime;
	double Parameters[MAX_PARAMETERS];
	
   if (NextNews(strNews, NewsTime, Parameters))
   {
      // create a straddle stop order if news will happen in 60 seconds
      if ( CurTime() >= NewsTime-60 && OpenStopOrders == 0 && OpenOrders == 0 )
      {
         CreateStraddle(Parameters);
      }
   }
   

THE FOLLOWING TWO ITEMS NEED TO BE SET UP
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

1. Global variable UTCtoServerTimeInMinutes is added to each instance of MT4 (see menu item Files->Global Variables)

It enables different instances of MT4 using different brokers to run EAs with identical time triggers

Examples:
   UTCtoServerTimeInMinutes = 0   for InterbankFX because their server time = UTC/GMT
   UTCtoServerTimeInMinutes = 180 for FXDD        because their server time = UTC/GMT plus 3 hours


2. File NewsItems.csv is added to the experts\files folder

Format of the file is as per the following example (the header must be included)

Description,Currency,DateTime,TimeZone,Parameter1,Parameter2,Parameter3
News description 1,USD,2006.08.09 12:00,UTC,0.5,0,3
News description 2,GBPUSD,2006.08.09 17:30,UTC,0,100,3
News occurring every day in US,USD,08:30,EST,0,50,4
News occurring every day in GB,GBP,09:30,BST,1,0,3

This format can be read and saved easily as a .csv file in excel

Valid values for each of the fields are

1. Description: string without commas

2. Currency: any 3 alpha currency string

3. DateTime: yyyy.mm.dd hh:mm  for a specific date 
     or      hh:mm  for a time that occurs every Day
     this second option can be a convenient way of ensuring that frequently used news times are always defined 
     without having to update the file. It will therefore work with backtesting.

4. TimeZone
   UTC: Time is in Greenwich Mean Time
   GMT: Time is in Greenwich Mean Time
   BST: Time is in British Summer Time (UTC + 1 hour - London with daylight saving)
   EST: Time is in Eastern Standard Time (UTC - 5 hours - New York)
   EDT: Time is in Eastern Daylight Time (UTC - 4 hours - New York with daylight saving)
   CST: Time is in South Australian CST Time (UTC + 9.5 hours)
   CSST: Time is in South Australian Central Summer Time (UTC + 10.5 hours)

5. Parameters
	Any number of parameters up to MAX_PARAMETERS

The news items can be cached to increase speed for backtesting by setting bCacheNews to true

*////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////


#property copyright "Paul Hampton-Smith"
#property link      ""

#include <stdlib.mqh>
#include <stderror.mqh>

string strNewsFileName = "NewsItems.csv"; // located in experts\files
int chNewsFileSeparator = ','; // to work in .csv format

#define MAX_NEWS_ITEMS  50
#define MAX_PARAMETERS  5

bool bNewsTrackerDebug = false; // switch on for lots more Print messages
static bool bNewsIsCached = false;
extern bool bCacheNews = false;


//////////////////////////////////////////////
// MAIN FUNCTIONS
//////////////////////////////////////////////


bool NewsEvent(int nWithinMinutes, string& strDescription, datetime& NewsTimeServer)
{
	// returns true if news event is nWithinMinutes
	// also returns description and news time
	
	double dblParameters[MAX_PARAMETERS];
   NextNews(strDescription, NewsTimeServer, dblParameters, nWithinMinutes);

   bool bReturn = CurTime() >= NewsTimeServer - nWithinMinutes*60 && 
               CurTime() <= NewsTimeServer + nWithinMinutes*60;
   if (bNewsTrackerDebug) Print("NewsEvent(",nWithinMinutes,") returning ",bToStr(bReturn)," and (\"",strDescription,"\",",TimeToStr(NewsTimeServer),")");
   return(bReturn);
}

bool NextNews(string& strDescription, datetime& EarliestNextTime, double& dblParameters[], int nMinutesInThePast = 0)
{
	// returns next news item details. If none, returns false

   static bool bDontBugMe = false;
   bool bReturn;
	int i;
	   
   static string strDescriptionArray[MAX_NEWS_ITEMS];
   static string strCurrencyArray[MAX_NEWS_ITEMS];
   static datetime dtServerArray[MAX_NEWS_ITEMS];
   static double dblParameters2D[MAX_NEWS_ITEMS][MAX_PARAMETERS];
   static int nNewsItemCount = 0;
   
   string strCurrency;
   datetime VeryFutureTime = StrToTime("2030.01.01 00:00");
   EarliestNextTime = VeryFutureTime;
   datetime DateTimeServer;

   if (!bNewsIsCached || !bCacheNews)
   {
   	nNewsItemCount = 0;
      int handle = FileOpen(strNewsFileName, FILE_CSV|FILE_READ, chNewsFileSeparator);
      if (handle <= 0)
      {   
         // only Alert the user once to the fact that strNewsFileName is missing
         if (!bDontBugMe)
         {
            if (IsTesting()) Alert("Please add file ",strNewsFileName," to tester\files folder, see NewsTracker.mqh for format");
            else Alert("Please add file ",strNewsFileName," to experts\files folder, see NewsTracker.mqh for format");
            bDontBugMe = true;
         }
         // repeated entries into log
         if (IsTesting()) Print("Please add file ",strNewsFileName," to tester\files folder, see NewsTracker.mqh for format");
         else Print("Please add file ",strNewsFileName," to experts\files folder, see NewsTracker.mqh for format");
         return(0);
      }
   
      // read header line
      while(!FileIsLineEnding(handle)) FileReadString(handle);
 
      // read news date lines and cache in arrays
      string strDescr, strCurr; datetime dtServer; double dblPar[MAX_PARAMETERS];
      bool bReadOK = ReadCSVline(handle, strDescr, strCurr, dtServer, dblPar);
      while (bReadOK)
      {
         strDescriptionArray[nNewsItemCount] = StringConcatenate(strCurr," - ",strDescr);
         strCurrencyArray[nNewsItemCount] = strCurr;
         dtServerArray[nNewsItemCount] = dtServer;
         for (i = 0 ; i < MAX_PARAMETERS ; i++)
         {
         	dblParameters2D[nNewsItemCount][i] = dblPar[i];
         }
			nNewsItemCount++;         
			bReadOK = ReadCSVline(handle, strDescr, strCurr, dtServer, dblPar);
      }
      FileClose(handle);
      if (bNewsTrackerDebug) Print("News is cached with ",nNewsItemCount," items");
		bNewsIsCached = true;     
   }

	// search for next news item for the currency pair
	int nEarliestNextTimeIndex = -1;
   for (i = 0 ; i < nNewsItemCount ; i++)
   {
      strCurrency = strCurrencyArray[i]; DateTimeServer = dtServerArray[i];
      if (CurTime()-nMinutesInThePast*60 > DateTimeServer || !CurrencyPairContains(strCurrency) ) continue;
      if (EarliestNextTime > DateTimeServer)
      {
         EarliestNextTime = DateTimeServer;
         nEarliestNextTimeIndex = i;
        	bReturn = true;
      }
   }

   if (nEarliestNextTimeIndex == -1)  
   {
      // No dates in the future
      EarliestNextTime = 0;
      DateTimeServer = 0;
      strDescription = "";
      bReturn = false;
   }
   else
   {
   	// found a future news item. Copy details to be returned
      strDescription = strDescriptionArray[nEarliestNextTimeIndex];
	   for (i = 0 ; i < MAX_PARAMETERS ; i++)
      {
      	dblParameters[i] = dblParameters2D[nEarliestNextTimeIndex][i];
      }
   }

   if (bNewsTrackerDebug) Print("Function NextNews(",nMinutesInThePast,") returning ",bToStr(bReturn)," and (",TimeToStr(EarliestNextTime),",\"",strDescription,"\",",ArrayTrace(dblParameters),")");
   return(bReturn);
}  





//////////////////////////////////////////////
// SUPPORTING FUNCTIONS
//////////////////////////////////////////////

bool ReadCSVline(int handle, string& strDescription, string& strCurrency, datetime& DateTimeServer, double& dblParameters[])
{
   int nLastError = 0;

//////////////////////////////////////
   // Handle

   if (handle <= 0)
   {
      Print("Invalid file handle for news file", strNewsFileName);
      if (bNewsTrackerDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }

//////////////////////////////////////
   // Description
   strDescription = FileReadString(handle); nLastError = GetLastError();
   if (nLastError != 0 && nLastError != ERR_NOT_INITIALIZED_STRING)
   {
      if (nLastError != ERR_END_OF_FILE) Print(ErrorDescription(nLastError)," reading description from ",strNewsFileName);
      if (bNewsTrackerDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }

//////////////////////////////////////
   // Currency
   strCurrency = FileReadString(handle); nLastError = GetLastError();
   if (nLastError != 0 && nLastError != ERR_NOT_INITIALIZED_STRING)
   {
      Print(ErrorDescription(nLastError)," reading currency from ",strNewsFileName);
      if (bNewsTrackerDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }

//////////////////////////////////////
   // DateTime
   string strDateTimeFromFile = FileReadString(handle); nLastError = GetLastError();
   if (nLastError != 0 && nLastError != ERR_NOT_INITIALIZED_STRING)
   {
      Print(ErrorDescription(nLastError)," reading DateTime from ",strNewsFileName);
      if (bNewsTrackerDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }
   
//////////////////////////////////////
   // Timezone
   string strTimeZone = FileReadString(handle); nLastError = GetLastError();
   if (nLastError != 0 && nLastError != ERR_NOT_INITIALIZED_STRING)
   {
      Print(ErrorDescription(nLastError)," reading TimeZone from ",strNewsFileName);
      if (bNewsTrackerDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }

//////////////////////////////////////
   // Convert DateTime
   datetime DateTimeFromFile = MyStrToTime(strDateTimeFromFile);
   if (DateTimeFromFile == 0)
   {
      Print("Problem with DateTime = \"",strDateTimeFromFile,"\" from ",strNewsFileName);
      if (bNewsTrackerDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }
   DateTimeServer = UTCtoServer(TimeToUTC(DateTimeFromFile, strTimeZone));
   
//////////////////////////////////////
   // Parameters
   int nParameterPos = 0;
   while(!FileIsLineEnding(handle)) 
   {
   	dblParameters[nParameterPos] = FileReadNumber(handle);
	   if (nLastError != 0)
   	{
      	Print(ErrorDescription(nLastError)," reading dblParameters[",nParameterPos,"] from ",strNewsFileName);
      	if (bNewsTrackerDebug) Print("Function ReadCSVline() returning false");
      	return(false);
   	}
   	nParameterPos++;
  	}

   if (bNewsTrackerDebug) Print("Function ReadCSVline() returning true and (\"",strDescription,"\",\"",strCurrency,"\",",TimeToStr(DateTimeServer),",",ArrayTrace(dblParameters),")");
   return(true);
}



string CleanFilename(string str)
{
	// change illegal filename characters in str to '-'
	int nPos; 

	string strRet = str;
	nPos = StringFind(str,"/");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');
	
	nPos = StringFind(strRet,"\\");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');
	
	nPos = StringFind(strRet,":");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,"*");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,"?");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, ' ');

	nPos = StringFind(strRet,"\"");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,">");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,"<");
	if (nPos != -1) strRet = StringSetChar(strRet, nPos, '-');

	nPos = StringFind(strRet,"|");
	if (nPos != -1) strRet= StringSetChar(strRet, nPos, '-');

	return(strRet);
}

string ArrayTrace(double arr[])
{
	// returns "arr[0],arr[1],..."
	string strRet = StringConcatenate(arr[0],"");
  	for (int i = 1 ; i < ArraySize(arr) ; i++)
  	{
   	strRet = StringConcatenate(strRet,",",arr[i]);
   }
   return(strRet);
}

string bToStr(bool b)
{
   if (b) return("true"); 
   else return("false");
}

datetime MyStrToTime(string strTime)
{
   // MT4 StrToTime doesn't add actual testing date to time when using "hh:mm" variation
 
   if (!IsTesting() || StringLen(strTime) > 5) 
   {
      // works fine when not testing and with yyyy.mm.dd hh:mm format
      return(StrToTime(strTime));
   }
   else
   {
      // use actual CurTime() date
      return(StrToTime(TimeToStr(CurTime(),TIME_DATE) + " " + strTime));
   }
}

string MyTimeToString(datetime dt)
{
   // output format dd/mm/yyyy mm:hh, eg for reading by Excel
   string strDayPad;
   int nDay = TimeDay(dt);
   if (nDay >= 0 && nDay <= 9) strDayPad = "0";
   else strDayPad = "";

   string strMonthPad;
   int nMonth = TimeMonth(dt);
   if (nMonth >= 0 && nMonth <= 9) strMonthPad = "0";
   else strMonthPad = "";
   
   string strHourPad;
   int nHour = TimeHour(dt);
   if (nHour >= 0 && nHour <= 9) strHourPad = "0";
   else strHourPad = "";
   
   string strMinutePad;
   int nMinute = TimeMinute(dt);
   if (nMinute >= 0 && nMinute <= 9) strMinutePad = "0";
   else strMinutePad = "";

   string strSecondPad;
   int nSecond = TimeSeconds(dt);
   if (nSecond >= 0 && nSecond <= 9) strSecondPad = "0";
   else strSecondPad = "";

   return(StringConcatenate(strDayPad,nDay,"/",strMonthPad,nMonth,"/",TimeYear(dt)," ",strHourPad,nHour,":",strMinutePad,nMinute,":",strSecondPad,nSecond));
}

datetime UTCtoServer(datetime UTC)
{
   // Uses global variable UTCtoServerTimeInMinutes to convert UTC to server time.
   // this enables different instances of MT4 using different brokers to run EAs with identical UTC time triggers
   // Examples:
   //    UTCtoServerTimeInMinutes = 0   for InterbankFX because their server time = UTC
   //    UTCtoServerTimeInMinutes = 180 for FXDD        because their server time = UTC plus 3 hours

   static bool bDontBugMe = false;
   if (!GlobalVariableCheck("UTCtoServerTimeInMinutes"))
   {
      // only Alert the user once to the fact that this variable is missing
      if (!bDontBugMe)
      {
         Alert("Please add global variable UTCtoServerTimeInMinutes (menu item Tools->Global Variables)");
         bDontBugMe = true;
      }
      // repeated entries into log
      Print("Please add global variable UTCtoServerTimeInMinutes (menu item Tools->Global Variables)");
      return(0);
   }
   if (bNewsTrackerDebug) Print("UTCtoServer(",TimeToStr(UTC),") returning ",TimeToStr(UTC+60*GlobalVariableGet("UTCtoServerTimeInMinutes")));
   return(UTC+60*GlobalVariableGet("UTCtoServerTimeInMinutes"));
}

datetime ServerToUTC(datetime dtServer)
{
   // Uses global variable UTCtoServerTimeInMinutes to convert server time to UTC.

   static bool bDontBugMe = false;
   if (!GlobalVariableCheck("UTCtoServerTimeInMinutes"))
   {
      // only Alert the user once to the fact that this variable is missing
      if (!bDontBugMe)
      {
         Alert("Please add global variable UTCtoServerTimeInMinutes (menu item Tools->Global Variables)");
         bDontBugMe = true;
      }
      // repeated entries into log
      Print("Please add global variable UTCtoServerTimeInMinutes (menu item Tools->Global Variables)");
      return(0);
   }
   if (bNewsTrackerDebug) Print("ServerToUTC(",TimeToStr(dtServer),") returning ",TimeToStr(dtServer-60*GlobalVariableGet("UTCtoServerTimeInMinutes")));
   return(dtServer-60*GlobalVariableGet("UTCtoServerTimeInMinutes"));
}

datetime TimeToUTC(datetime dt, string strTimeZone)
{
   int BST_TimeZone = 1;
   int EST_TimeZone = -4;
   int EDT_TimeZone = -5;
   double CST_TimeZone = 9.5;
   double CSST_TimeZone = 10.5;

   datetime dtReturn;
   if      (strTimeZone == "UTC") dtReturn = dt;
   else if (strTimeZone == "GMT") dtReturn = dt;
   else if (strTimeZone == "BST") dtReturn = dt - BST_TimeZone*3600;
   else if (strTimeZone == "EST") dtReturn = dt - EST_TimeZone*3600;
   else if (strTimeZone == "EDT") dtReturn = dt - EDT_TimeZone*3600;
   else if (strTimeZone == "CST") dtReturn = dt - CST_TimeZone*3600;
   else if (strTimeZone == "CSST") dtReturn = dt - CSST_TimeZone*3600;
   else 
   { 
      Print("Unknown timezone ",strTimeZone," in TimeToUTC()");
      dtReturn = 0;
   }
   
   if (bNewsTrackerDebug) Print("TimeToUTC(",TimeToStr(dt),",",strTimeZone,") returning ",TimeToStr(dtReturn));
   return(dtReturn);
}      

bool CurrencyPairContains(string strCurrency)
{
	bool bReturn; 
	if (StringLen(strCurrency) == 3)
	{ 
		// check if 3 alpha currency is either first or second part of pair
	   bReturn = StringSubstr(Symbol(),0,3) == strCurrency || StringSubstr(Symbol(),3,3) == strCurrency;
	}
	else
	{
		// check if currency pair is within Symbol()
	   bReturn = StringFind( Symbol(), strCurrency ) != -1;
	}
		
   if (bNewsTrackerDebug) Print("Function CurrencyPairContains(",strCurrency,") returning ",bToStr(bReturn));
   return(bReturn);
}

datetime ServerToLocal(datetime dtServer)
{
   int nRoundedMinutes = (LocalTime()-CurTime()+30)/60;
   return(dtServer + nRoundedMinutes*60);
}

extern bool bUtilsDebug;

int nUtilRetries = 5;
int nUtilOrderLock_mSec = 1000;
int nUtilRetryDelay = 500;

bool NewBarAfterTicks(int nAfterTicks)
{
   static int lastBars;
   static int nTicks = 0;
   nTicks++;
   if (lastBars != Bars && nTicks >= nAfterTicks)
   {
      lastBars = Bars;
      nTicks = 0;
      return(true);
   }
   else
   {
      return(false);
   }
}

bool NewBar()
{
   static int lastBars;
   if (lastBars != Bars)
   {
      lastBars = Bars;
      return(true);
   }
   else
   {
      return(false);
   }
}

int OpenOrders(int nSystemID = 0)
{
   int nCount = 0;
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; nPosition++ )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && // same symbol
         ( OrderType() == OP_BUY || OrderType() == OP_SELL ) &&
         (OrderMagicNumber() == nSystemID || nSystemID == 0) ) // same system
      {
         nCount++;
      }
   }
   return(nCount);
}

int OpenStopOrders(int nSystemID = 0)
{
   int nCount = 0;
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; nPosition++ )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && // same symbol
         ( OrderType() == OP_BUYSTOP || OrderType() == OP_SELLSTOP ) &&
         (OrderMagicNumber() == nSystemID || nSystemID == 0) ) // same system
      {
         nCount++;
      }
   }
   return(nCount);
}

void DeleteStopOrders(int nSystemID = 0)
{
   // note the shift of nPosition++ to lower in the for loop because it is not needed if an order has been deleted
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && // same symbol
         ( OrderType() == OP_BUYSTOP || OrderType() == OP_SELLSTOP ) &&
         (OrderMagicNumber() == nSystemID || nSystemID == 0) ) // same system
      {
         SecureOrderDelete(OrderTicket());
      }
      else
      {
         nPosition++;
      }
   }
}

int FindOrder(int cmd, int nSystemID)
{
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; nPosition++ )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && // same symbol
         ( OrderType() == cmd) &&
         OrderMagicNumber() == nSystemID) // same system
      {
         return(OrderTicket());
      }
   }
}

string DayOfWeekToString(int nDayOfWeek)
{
   switch(nDayOfWeek)
   {
      case 0: return("Sunday");
      case 1: return("Monday");
      case 2: return("Tuesday");
      case 3: return("Wednesday");
      case 4: return("Thursday");
      case 5: return("Friday");
      case 6: return("Saturday");
      default: Alert("Invalid day of week ",nDayOfWeek," in function DayOfWeekToString");
   }
   return("");
}

double ATRTrailingStop(int nOrderType, int nATRlen, double dblATRmult)
{
   double dblATR = iATR(OrderSymbol(),0,nATRlen,0);
   double dblStop, dblTightestStop;
   
   switch(nOrderType)
   {
      case OP_BUY:
         dblStop = Bid - dblATRmult * dblATR;
         // can't be any closer than tightest stop allowed on the market
         dblTightestStop = Bid - MarketInfo(Symbol(),MODE_STOPLEVEL) * Point;
         if ( dblStop > dblTightestStop ) dblStop = dblTightestStop;
         break;
         
      case OP_SELL:
         dblStop = Ask + dblATRmult * dblATR;
         // can't be any closer than tightest stop allowed on the market
         dblTightestStop = Ask + MarketInfo(Symbol(),MODE_STOPLEVEL) * Point;
         if ( dblStop < dblTightestStop ) dblStop = dblTightestStop;
         break;
      
      default: Alert("Invalid nOrdertype ",nOrderType," in function ATRTrailingStop");
   }
   return(dblStop);
}

double MEMATrailingStop(int nOrderType, int nATRlen, double dblATRmult, double dblAccel)
{
   double dblATR = iATR(OrderSymbol(),0,nATRlen,0);
   double dblStopLoss = OrderStopLoss();
   double dblNewStopLoss, dblTightestStop;
   double tmp; 
   
   switch(nOrderType)
   {
      case OP_BUY:
         // can't be any closer than tightest stop allowed on the market
         dblTightestStop = Bid - MarketInfo(Symbol(),MODE_STOPLEVEL) * Point;
 
     		tmp = High[0] - dblATRmult*dblATR - dblStopLoss;
     		if (tmp > 0.0)
     		{
            dblNewStopLoss = dblStopLoss + dblAccel*tmp;
     		}
         if ( dblNewStopLoss > dblTightestStop ) dblNewStopLoss = dblTightestStop;
         break;
         
      case OP_SELL:
         // can't be any closer than tightest stop allowed on the market
         dblTightestStop = Ask + MarketInfo(Symbol(),MODE_STOPLEVEL) * Point;
 
     		tmp = Low[0] + dblATRmult*dblATR - dblStopLoss;
     		if (tmp < 0.0)
     		{
            dblNewStopLoss = dblStopLoss + dblAccel*tmp;
     		}
         if ( dblNewStopLoss < dblTightestStop ) dblNewStopLoss = dblTightestStop;
         break;
      
      default: Alert("Invalid nOrdertype ",nOrderType," in function ATRTrailingStop");
   }
   return(dblNewStopLoss);
}

      
int CompletedOrdersSince(datetime dt, int nSystemID)
{
   int nCount = 0;
   for ( int nPosition=0 ; nPosition<HistoryTotal() ; nPosition++ )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_HISTORY);
      if ( OrderSymbol()==Symbol() && // same symbol
         OrderMagicNumber() == nSystemID &&  // same system
         ( OrderType() == OP_BUY || OrderType() == OP_SELL ) && 
         OrderCloseTime() >= dt)
      {
         nCount++;
      }
   }
   return(nCount);
}

string TimeString(int hour, int minute)
{
   // used to create "05" instead of "5"
   if (minute<0 || minute>59 || hour<0 || hour>23) Alert("Incorrect parameters ",hour,":",minute," in TimeString");
   
   string strHourPad;
   if (hour >= 0 && hour <= 9) strHourPad = "0";
   else strHourPad = "";
   
   string strMinutePad;
   if (minute >= 0 && minute <= 9) strMinutePad = "0";
   else strMinutePad = "";

   return(StringConcatenate(strHourPad,hour,":",strMinutePad,minute));
}

double LastClosedOrderProfit(int nOrderType, int nSystemID)
{
   int nCount = 0;
   for ( int nPosition=HistoryTotal() ; nPosition>=0 ; nPosition-- )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_HISTORY);
      if ( OrderSymbol()==Symbol() && // same symbol
         OrderMagicNumber() == nSystemID &&  // same system
         ( OrderType() == OP_BUY || OrderType() == OP_SELL ) )
       
      {
         return(OrderProfit());
      }
   }
   return(0.0);
}

double HighLowTrailingStop(bool bLong, int nLookBack)
{
   double dblStop, dblTightestStop;
   if (bLong)
   {
      // Lowest low, minus one point to stay away from support lines
      dblStop = Low[ Lowest(OrderSymbol(),0,MODE_LOW,nLookBack) ] - Point;
      // can't be any closer than tightest stop
      dblTightestStop = Bid - MarketInfo(Symbol(),MODE_STOPLEVEL) * Point;
      if ( dblStop > dblTightestStop ) dblStop = dblTightestStop;
   }
   else
   { // short
      // Highest high, plus one point to stay away from support lines
      dblStop = High[ Highest(OrderSymbol(),0,MODE_HIGH,nLookBack) ] + Point;
      // can't be any closer than tightest stop
      dblTightestStop = Ask + MarketInfo(Symbol(),MODE_STOPLEVEL) * Point;
      if ( dblStop < dblTightestStop ) dblStop = dblTightestStop;
   }
   return(dblStop);
}

void CloseAllAtEndOfWeek(int nSystemID = 0)
{
   // exit all if time is 17:00 UTC Fridays
   if ( !(TimeDayOfWeek(Time[0]) == 5 && TimeHour(Time[0]) == 17) ) return;
   
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && // same symbol
         (OrderMagicNumber() == nSystemID || nSystemID == 0) &&  // same system
         ( OrderType() == OP_BUY || OrderType() == OP_SELL ) )
      {
         Print("End of week close ", TimeToStr(Time[0]));
         switch(OrderType())
         {
         case OP_BUY:
            OrderClose(OrderTicket(), OrderLots(), Bid, 0, Purple);
            return;
               
         case OP_SELL:
            OrderClose(OrderTicket(), OrderLots(), Ask, 0, Purple);
            return;

         case OP_BUYSTOP:
            OrderDelete(OrderTicket());
            return;

         case OP_SELLSTOP:
            OrderDelete(OrderTicket());
            return;
         } // switch
      }
      else
      {
         nPosition++;
      }
   }
}

datetime WeeklyExpirationTime(int nExitDay, int nExitHour, int nExitMinute)
{
   datetime dtNow = CurTime();
   datetime dtMidnightToday = (dtNow/86400)*86400;
   int nDaysToExit = nExitDay - TimeDayOfWeek(dtNow);

   datetime dtExitTime = dtMidnightToday + nDaysToExit*86400 + nExitHour*3600 + nExitMinute*60;
   return(dtExitTime);
}

void AdjustTrailingStops(int nTrailingStop, int nSystemID = 0)
{
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; nPosition++ )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == 0 || OrderMagicNumber() == nSystemID) ) // same symbol and system
      {
         switch(OrderType())
         {
         case OP_BUY:
            // adjust long trailing stop up if possible
            if (Bid - nTrailingStop*Point > OrderStopLoss()+0.1*Point )
            {
               OrderModify(OrderTicket(),OrderOpenPrice(),Ask - nTrailingStop*Point,OrderTakeProfit(),0,Green);
            }
            break;
               
         case OP_SELL:
            // adjust short trailing stop down if possible
            if (Ask + nTrailingStop*Point-0.1*Point < OrderStopLoss() || OrderStopLoss() == 0.0 )
            {
               OrderModify(OrderTicket(),OrderOpenPrice(),Bid + nTrailingStop*Point,OrderTakeProfit(),0,Red);
            }
         } // switch
      } 
   }
}

bool OrderOfTypeExists(int nOrderType, int nSystemID = 0)
{
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; nPosition++ )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol()
         && (OrderMagicNumber() == 0 || OrderMagicNumber() == nSystemID)
         && OrderType() == nOrderType ) // same symbol, system & type
      {
         return(true);
      }
   }
}
   
void CloseOrdersAfterDuration(int nSeconds, int nSystemID = 0)
{
   for ( int nPosition=0 ; nPosition<OrdersTotal() ;  )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && 
         (OrderMagicNumber() == nSystemID || nSystemID == 0) &&
         Time[0] > OrderOpenTime() + nSeconds)
      {
         switch (OrderType())
         {
         case OP_BUY: SecureOrderClose(OrderTicket(), OrderLots(), Bid, 5, Purple); break;
         case OP_SELL: SecureOrderClose(OrderTicket(), OrderLots(), Ask, 5, Purple); break;
         }
      }
      else
      {
         nPosition++;
      }
   }
}

void CloseOrdersAtTime(datetime dt, int nSystemID = 0)
{
   // note the shift of nPosition++ to lower in the for loop because it is not needed if an order has been closed
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && 
         (OrderMagicNumber() == nSystemID || nSystemID == 0) &&
         CurTime() > dt)
      {
         switch (OrderType())
         {
         case OP_BUY: SecureOrderClose(OrderTicket(), OrderLots(), Bid, 5, Purple); break;
         case OP_SELL: SecureOrderClose(OrderTicket(), OrderLots(), Ask, 5, Purple); break;
         }
      }
      else
      {
         nPosition++;
      }
   }
}

void DeleteStopOrdersAtTime(datetime dt, int nSystemID = 0)
{
   // note the shift of nPosition++ to lower in the for loop because it is not needed if an order has been deleted
   for ( int nPosition=0 ; nPosition<OrdersTotal() ; )
   {
      SecureOrderSelect(nPosition, SELECT_BY_POS);
      if ( OrderSymbol()==Symbol() && 
         (OrderMagicNumber() == nSystemID || nSystemID == 0) &&
         CurTime() > dt &&
         (OrderType() == OP_BUYSTOP || OrderType() == OP_SELLSTOP))
      {
         SecureOrderDelete(OrderTicket());
      }
      else
      {
         nPosition++;
      }
   }
}

int SecureOrderSend( string symbol, int cmd, double volume, double price, int slippage, double stoploss, double takeprofit, string comment, int magic=0, datetime expiration=0, color arrow_color=CLR_NONE)
{
   if (cmd == OP_SELL || cmd == OP_SELLLIMIT || cmd == OP_BUY || cmd == OP_BUYLIMIT)
   {
      Alert(CmdToStr(cmd)," not implemented yet in SecureOrderSend");
      return(-1);
   }
   
   for (int nTry = 0 ; nTry < nUtilRetries ; nTry++)
   {
      double dblMinStopDistance = MarketInfo(Symbol(),MODE_STOPLEVEL)*Point;
      if (cmd == OP_BUYSTOP)
      {
         price = MathMax(Ask+dblMinStopDistance+Point,price);
         if (stoploss == 0)
            stoploss = price-dblMinStopDistance-Point;
         else
            stoploss = MathMin(price-dblMinStopDistance-Point, stoploss);
         
      }
      else if (cmd == OP_SELLSTOP)
      {
         if (price == 0) 
            price = Bid-dblMinStopDistance-Point;
         else 
            price = MathMin(Bid-dblMinStopDistance-Point,price);
         stoploss = MathMax(price+dblMinStopDistance+Point, stoploss);
      }
      
      price = NormalizeDouble(price,Digits);
      stoploss = NormalizeDouble(stoploss,Digits);
      takeprofit = NormalizeDouble(takeprofit,Digits);
      OrderLock(nUtilOrderLock_mSec);
      int nTicket = OrderSend( symbol, cmd, volume, price, slippage, stoploss, takeprofit, comment, magic, expiration, arrow_color);
      int nLastError = GetLastError();
		OrderLockClear();
      Print(ErrorDescription(nLastError)," for SecureOrderSend(",CmdToStr(cmd),"): Ask ",Ask," Bid ",Bid," MinStop ",dblMinStopDistance," price ", price," stoploss ",stoploss," takeprofit ", takeprofit);
      if (nLastError == 0) return(nTicket);
      Sleep(nUtilRetryDelay);
   }
   return(-1);
}

bool SecureOrderClose( int ticket, double lots, double price, int slippage, color Color=CLR_NONE)
{
   for (int nTry = 0 ; nTry < nUtilRetries ; nTry++)
   {
   	OrderLock(nUtilOrderLock_mSec);
      OrderClose(ticket, lots, price, slippage, Color);
      int nLastError = GetLastError();
		OrderLockClear();
      Print(ErrorDescription(nLastError)," for SecureOrderClose. price ",price);
      if (nLastError == 0) return(true);
      Sleep(nUtilRetryDelay);
   }
   return(false);
}

bool SecureOrderDelete(int ticket)
{
   for (int nTry = 0 ; nTry < nUtilRetries ; nTry++)
   {
   	OrderLock(nUtilOrderLock_mSec);
      OrderDelete(ticket);
      int nLastError = GetLastError();
		OrderLockClear();
      Print(ErrorDescription(nLastError)," for SecureOrderDelete");
      if (nLastError == 0) return(true);
      Sleep(nUtilRetryDelay);
   }
   return(false);
}

bool SecureOrderSelect( int index, int select, int pool=MODE_TRADES) 
{
   for (int nTry = 0 ; nTry < nUtilRetries ; nTry++)
   {
      OrderSelect( index, select, pool);
      int nLastError = GetLastError();
      if (nLastError == 0 && OrderTicket() != 0) return(true);
      else Print(ErrorDescription(nLastError)," for SecureOrderSelect");
   }
   return(false);
}

string CmdToStr(int cmd)
{
   switch(cmd)
   {
   case OP_BUY: return("OP_BUY");
   case OP_SELL: return("OP_SELL");
   case OP_BUYSTOP: return("OP_BUYSTOP");
   case OP_SELLSTOP: return("OP_SELLSTOP");
   case OP_BUYLIMIT: return("OP_BUYLIMIT");
   case OP_SELLLIMIT: return("OP_SELLLIMIT");
   default: return("OP_UNKNOWN");
   }
}      
       
double ClosestStopLoss(int cmd, double dblPrice)
{
   double dblMinStopDistance = MarketInfo(Symbol(),MODE_STOPLEVEL)*Point;
   
   switch(cmd)
   {
   case OP_BUY:
   case OP_BUYLIMIT:
   case OP_BUYSTOP:
      return(NormalizeDouble(dblPrice-dblMinStopDistance,Digits));
      
   case OP_SELL:
   case OP_SELLLIMIT:
   case OP_SELLSTOP:
      return(NormalizeDouble(dblPrice+dblMinStopDistance,Digits));

   default:
      Alert("Invalid cmd ",cmd," in MinStopLoss");
   }
}
       
double ClosestBuySellStopPrice(int cmd)
{
   double dblMinStopDistance = MarketInfo(Symbol(),MODE_STOPLEVEL)*Point;
   
   switch(cmd)
   {
   case OP_BUYSTOP:
      return( MarketInfo(Symbol(),MODE_ASK)+dblMinStopDistance );
      
   case OP_SELLSTOP:
      return( MarketInfo(Symbol(),MODE_BID)-dblMinStopDistance );
      
   default:
      Alert("Invalid cmd ",CmdToStr(cmd)," in ClosestBuySellStopPrice");
   }
   return(0);
}
    
    
void CloseAfterPeak(int nTrailingStop, int nSlippage, bool bReverse = false, int nSystemID = 0, datetime AfterTime = 0)
{
   string strComment;
   double dblLots;
   int nInitialStop, nPosition;   
   double dblInitialStop, dblProfitTarget;
   
   // do nothing until AfterTime
   if (AfterTime != 0 && CurTime() <= AfterTime) return;

   // client side stoploss. If internet connection is broken this will fail! 
   // Ensure that initial stop is also set to control risk
   static double dblMinAsk = 999;
   static double dblMaxBid = 0;

   for ( nPosition=0 ; nPosition<OrdersTotal() ; nPosition++ )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
      if ( OrderSymbol()==Symbol() && (OrderMagicNumber() == 0 || OrderMagicNumber() == nSystemID) ) // same symbol and system
      {
         dblMaxBid = MathMax(dblMaxBid, Bid);
         dblMinAsk = MathMin(dblMinAsk, Ask);
            
         switch(OrderType())
         {
         case OP_BUY:
            if (bUtilsDebug) Print("CloseAfterPeak(",nTrailingStop,",",nSlippage,",",bReverse,",",nSystemID,",",TimeToStr(AfterTime),") with dblMaxBid ",dblMaxBid," Bid ",Bid,". ",MathRound((dblMaxBid-Bid+Point*0.1)/Point)," pips from peak");
            if (Bid <= dblMaxBid - nTrailingStop*Point+0.1*Point)
            {
               if (bReverse)
               {
                  strComment = OrderComment();
                  dblLots = OrderLots();
                  nInitialStop = (OrderOpenPrice()-OrderStopLoss()+0.1*Point)/Point;
               }
               
               Print("Close Buy order after peak of ",dblMaxBid," Bid ",Bid," TrailingStop ", nTrailingStop);
               if (SecureOrderClose( OrderTicket(), OrderLots(), Bid, nSlippage, Turquoise)) 
               {
                  if (bReverse)
                  {
                     dblInitialStop = Ask + nInitialStop*Point;
                     dblProfitTarget = 0;
                     OrderSend(Symbol(),OP_SELL,dblLots,Bid,nSlippage,dblInitialStop,dblProfitTarget,strComment,nSystemID,0,Red);
                     dblMaxBid = Bid;
                     dblMinAsk = Ask;
                  }
                  else
                  {
                     dblMaxBid = 0;
                     dblMinAsk = 999;
                  }
               }
            }
            break;
               
         case OP_SELL:
            if (bUtilsDebug) Print("CloseAfterPeak(",nTrailingStop,",",nSlippage,",",bReverse,",",nSystemID,",",TimeToStr(AfterTime),") with dblMinAsk ",dblMinAsk," Ask ",Ask,". ",MathRound((Ask-dblMinAsk+Point*0.1)/Point)," pips above negative peak");
            if (Ask+0.1*Point >= dblMinAsk + nTrailingStop*Point)
            {
               if (bReverse)
               {
                  strComment = OrderComment();
                  dblLots = OrderLots();
                  nInitialStop = (OrderStopLoss()-OrderOpenPrice()+0.1*Point)/Point;
               }

               Print("Close Sell order after negative peak of ",dblMinAsk," Ask ",Ask," TrailingStop ", nTrailingStop);
               if (SecureOrderClose( OrderTicket(), OrderLots(), Ask, nSlippage, Coral))
               {
                  if (bReverse)
                  {
                     dblInitialStop = Bid - nInitialStop*Point;
                     dblProfitTarget = 0;
                     OrderSend(Symbol(),OP_BUY,dblLots,Ask,nSlippage,dblInitialStop,dblProfitTarget,strComment,nSystemID,0,Green);
                     dblMaxBid = Bid;
                     dblMinAsk = Ask;
                  }
                  else
                  {
                     dblMaxBid = 0;
                     dblMinAsk = 999;
                  }
               }
            }
            break;
         }
      }
   }
}
   

void InitialOpenAfterPeak(double dblLots, int nInitialStop, int nTrailingStop, int nSlippage, string strComment, int nSystemID = 0, datetime AfterTime = 0)
{
   // opens SELL order if no orders exist and price falls nTrailingStop from peak 
   // opens BUY order if no orders exist and price rises nTrailingStop from negative peak 

   double dblInitialStop, dblProfitTarget;
   
   // do nothing until AfterTime
   if ((AfterTime != 0 && CurTime() <= AfterTime) || OpenOrders(nSystemID) > 0) return;

   static double dblMinAsk = 999;
   static double dblMaxBid = 0;
   
   dblMaxBid = MathMax(dblMaxBid, Bid);
   dblMinAsk = MathMin(dblMinAsk, Ask);

   if (Bid <= dblMaxBid - nTrailingStop*Point+0.1*Point)
   {
      dblInitialStop = Ask + nInitialStop*Point;
      dblProfitTarget = 0;
      OrderSend(Symbol(),OP_SELL,dblLots,Bid,nSlippage,dblInitialStop,dblProfitTarget,strComment,nSystemID,0,Red);
      Print(ErrorDescription(GetLastError()));
      dblMinAsk = 999; dblMaxBid = 0;
   }
   else if (Ask+0.1*Point >= dblMinAsk + nTrailingStop*Point)
   {
      dblInitialStop = Bid - nInitialStop*Point;
      dblProfitTarget = 0;
      OrderSend(Symbol(),OP_BUY,dblLots,Ask,nSlippage,dblInitialStop,dblProfitTarget,strComment,nSystemID,0,Green);
      Print(ErrorDescription(GetLastError()));
      dblMinAsk = 999; dblMaxBid = 0;
   }
   if (bUtilsDebug) Print("InitialOpenAfterPeak(",dblLots,",",nInitialStop,",",nTrailingStop,",",nSlippage,",",strComment,",",nSystemID,",",TimeToStr(AfterTime),") with dblMinAsk ",dblMinAsk," Ask ",Ask," dblMaxBid ",dblMaxBid," Bid ",Bid);
}


bool MaxLosingOrdersAfterTime(datetime dt, int nMaxLosing, bool bIncludeOpenOrders, int nSystemID)
{
   int nCount = 0;

   if (bIncludeOpenOrders)
   {   
      // first look at open orders
      for ( int nPosition=0 ; nPosition < OrdersTotal() ; nPosition++ )
      {
         OrderSelect(nPosition, SELECT_BY_POS, MODE_TRADES);
         if ( OrderSymbol()==Symbol() && // same symbol
            OrderMagicNumber() == nSystemID &&  // same system
            ( OrderType() == OP_BUY || OrderType() == OP_SELL ) && 
            OrderOpenTime() >= dt &&  // after time
            OrderProfit() < 0) // losing trades
         {
            nCount++;
            if (nCount >= nMaxLosing) return(true);
         }
      }
   }
   
   // then closed orders
   for ( nPosition=HistoryTotal()-1 ; nPosition>=0 ; nPosition-- )
   {
      OrderSelect(nPosition, SELECT_BY_POS, MODE_HISTORY);
      if ( OrderSymbol()==Symbol() && // same symbol
         OrderMagicNumber() == nSystemID &&  // same system
         ( OrderType() == OP_BUY || OrderType() == OP_SELL ) && 
         OrderOpenTime() >= dt &&  // after time
         OrderProfit() < 0) // losing trades
      {
         nCount++;
         if (nCount >= nMaxLosing) return(true);
      }
   }
   return(false);
}

double Lots(double dblLots, int nPercentOfMargin)
{
	double dblRet;
   if (dblLots == 0.0)
   {
  		dblRet = (Truncate((nPercentOfMargin/100.0)*AccountFreeMargin()*AccountLeverage()/100000,1));
		// if using 100%, reduce slightly to ensure under limit
   	if (nPercentOfMargin == 100) dblRet -= 0.1;
   }
   else
   {
   	dblRet = dblLots;
   }
   if (bUtilsDebug) Print("Lots(",dblLots,",",nPercentOfMargin,") returning ",dblRet);
	// minimum lots is 0.1
   return(MathMax(dblRet,0.1));
}

double Truncate(double dblVal, int nDigits)
{
   double dblPower = MathPow(10,nDigits);
   int nVal = dblVal*dblPower;
   return(nVal/dblPower);
}


void OrderLock(int nTime_mSec)
{
	if (!GlobalVariableCheck("OrderLock"))
	{
		GlobalVariableSet("OrderLock",CurTime());
	}
	else
	{
		int nLoopLen = nTime_mSec/100;
		int nCount = 0;
		while (nCount < nLoopLen && GlobalVariableCheck("OrderLock") ) 
		{
			Sleep(100);
			nCount++;
		}
	}
}

void OrderLockClear()
{
	GlobalVariableDel("OrderLock");
}

